/*******************************************************************************
NAME                     ALBERS CONICAL EQUAL-AREA 

PURPOSE:	Transforms input Easting and Northing to longitude and
		latitude for the Albers Conical Equal Area projection.  The
		Easting and Northing must be in meters.  The longitude
		and latitude values will be returned in radians.

PROGRAMMER              DATE
----------              ----
T. Mittan,       	Feb, 1992
S. Nelson		Feb, 1996	Made a modification to the
					assignment to "con" enclosing the
					section 1.0 - e3 in parenthesis.
A. Koning		May 18, 2004	Ported to javascript

ALGORITHM REFERENCES

1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
*******************************************************************************/

/* Variables common to all subroutines in this code file
  -----------------------------------------------------*/
	var r_major;		/* major axis                           */
	var r_minor;        /* minor axis                           */
	var c;              /* constant c                           */
	var e3;             /* eccentricity                         */
	var es;				/* eccentricity squared					*/
	var rh;             /* height above elipsoid                */
	var ns0;            /* ratio between meridians              */
	var lon_center;     /* center longitude                     */
	var false_easting;  /* x offset in meters                   */
	var false_northing; /* y offset in meters                   */

/* Initialize the Albers projection
  -------------------------------*/
function alberinvint(r_maj,r_min,lat1,lat2,lon0,lat0,false_east,false_north) {
	/*---
	r_maj: 					major axis                           
	r_min:                  minor axis                           
	lat1:                   first standard parallel              
	lat2:                   second standard parallel             
	lon0:                   center longitude                     
	lat0:                   center lattitude                     
	false_east:             x offset in meters                   
	false_north:            y offset in meters                   

	sin_po,cos_po:			sine and cos values					
	con:					temporary variable					
	temp:					temporary variable					
	ms1:   		        	small m 1                            
	ms2:            		small m 2                            
	qs0:            		small q 0                            
	qs1:            		small q 1                            
	qs2:            		small q 2                            
	---*/
	
	false_easting = false_east;
	false_northing = false_north;
	lon_center = lon0;
	if (Math.abs(lat1 + lat2) < EPSLN) {
		//alert("Equal latitudes for Standard Parallels on opposite sides of equator,alber-invinit");
		return(31);
	}
	r_major = r_maj;
	r_minor = r_min;
	temp = r_minor / r_major;
	es = 1.0 - Math.pow(temp,2);
	e3 = Math.sqrt(es);

	sincos_po = sincos(lat1);
	sin_po = sincos_po[0];
	cos_po = sincos_po[1];
	con = sin_po;

	ms1 = msfnz(e3,sin_po,cos_po);
	qs1 = qsfnz(e3,sin_po,cos_po);

	sincos_po = sincos(lat2);
	sin_po = sincos_po[0];
	cos_po = sincos_po[1];

	ms2 = msfnz(e3,sin_po,cos_po);
	qs2 = qsfnz(e3,sin_po,cos_po);

	sincos_po = sincos(lat0);
	sin_po = sincos_po[0];
	cos_po = sincos_po[1];

	qs0 = qsfnz(e3,sin_po,cos_po);

	if (Math.abs(lat1 - lat2) > EPSLN)
		ns0 = (ms1 * ms1 - ms2 * ms2) / (qs2 - qs1);
	else
		ns0 = con;
	c = ms1 * ms1 + ns0 * qs1;
	rh = r_major * Math.sqrt(c - ns0 * qs0) / ns0;

	return true;
}

/* Albers Conical Equal Area inverse equations--mapping x,y to lat/long
  -------------------------------------------------------------------*/
function alberinv(x, y) {
	/*---
	x:					(O) X projection coordinate
	y:					(O) Y projection coordinate
	
	rh1:				height above ellipsoid
	qs:					function q
	con:				temporary sign value
	theta:				angle
	---*/

	FLAG = 0;
	x -= false_easting;
	y = rh - y + false_northing;;
	if (ns0 >= 0) {
		rh1 = Math.sqrt(x * x + y * y);
		con = 1.0;
	}
	else {
		rh1 = -Math.sqrt(x * x + y * y);
		con = -1.0;
	}
	theta = 0.0;
	if (rh1 != 0.0)
		theta = Math.atan2(con * x, con * y);
	con = rh1 * ns0 / r_major;
	qs = (c - con * con) / ns0;
	if (e3 >= 1e-10) {
		con = 1 - .5 * (1.0 - es) * Math.log((1.0 - e3) / (1.0 + e3)) / e3;
		if (Math.abs(Math.abs(con) - Math.abs(qs)) > .0000000001 ) {
			lat = phi1z(e3,qs,FLAG);
			if (FLAG != 0)
				return(FLAG);
		}
		else {
			if (qs >= 0)
				lat = .5 * PI;
			else
				lat = -.5 * PI;
		}
	}
	else {
		lat = phi1z(e3,qs,FLAG);
		if (FLAG != 0)
			return;
	}

	lon = adjust_lon(theta/ns0 + lon_center);

	lonlat = new Array(lon,lat);
	return lonlat;
}
