/*******************************************************************************
NAME                Projection support routines listed below.

PURPOSE:	The following functions are included in CPROJ.C.

		SINCOS:	  Calculates the sine and cosine.
		ASINZ:	  Eliminates roundoff errors.
		MSFNZ:	  Computes the constant small m for Oblique Equal Area.
		QSFNZ:	  Computes the constant small q for Oblique Equal Area.
		PHI1Z:	  Computes phi1 for Albers Conical Equal-Area.
		PHI2Z:	  Computes the latitude angle, phi2, for Lambert
			  Conformal Conic and Polar Stereographic.
		PHI3Z:	  Computes the latitude, phi3, for Equidistant Conic.
		PHI4Z:	  Computes the latitude, phi4, for Polyconic.
		PAKCZ:	  Converts a 2 digit alternate packed DMS format to
			  standard packed DMS format.
		PAKR2DM:  Converts radians to 3 digit packed DMS format.
		TSFNZ:	  Computes the small t for Lambert Conformal Conic and
			  Polar Stereographic.
		SIGN:	  Returns the sign of an argument.
		ADJUST_LON:  Adjusts a longitude angle to range -180 to 180.
		E0FN, E1FN, E2FN, E3FN:
			  Computes the constants e0,e1,e2,and e3 for
			  calculating the distance along a meridian.
		E4FN:	  Computes e4 used for Polar Stereographic.
		MLFN:	  Computes M, the distance along a meridian.
		CALC_UTM_ZONE:	Calculates the UTM zone number.

PROGRAMMER              DATE		REASON
----------              ----		------
D. Steinwand, EROS      July, 1991	Initial development
T. Mittan, EROS		May, 1993	Modified from Fortran GCTP for C GCTP
S. Nelson, EROS		June, 1993	Added inline comments
S. Nelson, EROS		Nov, 1993	Added loop counter in ADJUST_LON
S. Nelson, EROS		Jan, 1998	Changed misspelled error message
A. Koning, UNBC		May 18, 2004 Ported to javascript

*******************************************************************************/
var FLAG;
var MAX_VAL  = 4;
var MAXLONG  = 2147483647.;
var DBLLONG  = 4.61168601 * Math.pow(10,18);

// from proj.h
var PI = 3.141592653589793238;
var HALF_PI = (PI*0.5);
var TWO_PI = (PI*2.0);
var EPSLN = 1.0 * Math.pow(10,-10);
var R2D = 57.2957795131;
var D2R = 1.74532925199432 * Math.pow(10,-2);
var S2R = 4.848136811095359 * Math.pow(10,-6);

var OK = 0;
var ERROR = -1;
var IN_BREAK = -2;
// ------

/* Function to calculate the sine and cosine in one call.  Some computer
   systems have implemented this function, resulting in a faster implementation
   than calling each function separately.  It is provided here for those
   computer systems which don`t implement this function
  ----------------------------------------------------*/
function sincos(val) {
	sin_val = Math.sin(val);
	cos_val = Math.cos(val);
	sincos_val = new Array(sin_val,cos_val);
	return sincos_val;
}

/* Function to eliminate roundoff errors in asin
----------------------------------------------*/
function asinz (con) {
	if (Math.abs(con) > 1.0) {
		if (con > 1.0)
			con = 1.0;
		else
			con = -1.0;
	}
	return(Math.asin(con));
}

/* Function to compute the constant small m which is the radius of
   a parallel of latitude, phi, divided by the semimajor axis.
---------------------------------------------------------------*/
function msfnz (eccent,sinphi,cosphi) {
	con = eccent * sinphi;
	return((cosphi / (Math.sqrt (1.0 - con * con))));
}

/* Function to compute constant small q which is the radius of a 
   parallel of latitude, phi, divided by the semimajor axis. 
------------------------------------------------------------*/
function qsfnz (eccent,sinphi,cosphi) {
	if (eccent > 1.0 * Math.pow(10,-7)) {
		con = eccent * sinphi;
		return (( 1.0- eccent * eccent) * (sinphi /(1.0 - con * con) - (.5/eccent) * Math.log((1.0 - con)/(1.0 + con))));
	}
	else
		return(2.0 * sinphi);
}

/* Function to compute phi1, the latitude for the inverse of the
   Albers Conical Equal-Area projection.
-------------------------------------------*/
function phi1z (eccent,qs) {
	phi = asinz(.5 * qs);
	if (eccent < EPSLN)
		return(phi);
	eccnts = eccent * eccent; 
	for (i = 1; i <= 25; i++) {
		//sincos(phi,&sinpi,&cospi);
		// added ----- \
		sincospi = sincos(phi);//, sinpi, cospi);
		sinpi = sincospi[0];
		cospi = sincospi[1];
		// added ----- /
		con = eccent * sinpi; 
		com = 1.0 - con * con;
		dphi = .5 * com * com / cospi * (qs / (1.0 - eccnts) - sinpi / com + .5 / eccent * Math.log ((1.0 - con) / (1.0 + con)));
		phi = phi + dphi;
		if (Math.abs(dphi) <= 1.0 * Math.pow(10,-7))
			return(phi);
	}
	//alert("Convergence error,phi1z-conv");
	FLAG = 001;
	return false;
}

/* Function to compute the latitude angle, phi2, for the inverse of the
   Lambert Conformal Conic and Polar Stereographic projections.
----------------------------------------------------------------*/
function phi2z(eccent,ts) {
	FLAG = 0;
	eccnth = .5 * eccent;
	phi = HALF_PI - 2 * Math.atan(ts);
	for (i = 0; i <= 15; i++) {
		sinpi = Math.sin(phi);
		con = eccent * sinpi;
		dphi = HALF_PI - 2 * Math.atan(ts * (Math.pow(((1.0 - con)/(1.0 + con)),eccnth))) - 
			phi;
		phi += dphi; 
		if (Math.abs(dphi) <= .0000000001)
			return(phi);
	}
	//alert("Convergence error,phi2z-conv");
	FLAG = 002;
	return(002);
}
 
/* Function to compute latitude, phi3, for the inverse of the Equidistant
   Conic projection.
-----------------------------------------------------------------*/
function phi3z(ml,e0,e1,e2,e3) {
	phi = ml;
	
	for (i = 0; i < 15; i++) {
		dphi = (ml + e1 * Math.sin(2.0 * phi) - e2 * Math.sin(4.0 * phi) + e3 * Math.sin(6.0 * phi))
			/ e0 - phi;
		phi += dphi;
		if (Math.abs(dphi) <= .0000000001) {
			FLAG = 0;
			return(phi);
		}
	}
	//alert("Latitude failed to converge after 15 iterations,PHI3Z-CONV");
	FLAG = 3;
	return(3);
}

/* Function to compute, phi4, the latitude for the inverse of the
   Polyconic projection.
------------------------------------------------------------*/
function phi4z (eccent,e0,e1,e2,e3,a,b) {
	phi = a;
	for (i = 1; i <= 15; i++) {
		sinphi = Math.sin(phi);
		tanphi = Math.tan(phi);
		c = tanphi * Math.sqrt (1.0 - eccent * sinphi * sinphi);
		sin2ph = Math.sin (2.0 * phi);
		ml = e0 * phi - e1 * sin2ph + e2 * Math.sin (4.0 *  phi) - e3 * 
		sin (6.0 *  phi);
		mlp = e0 - 2.0 * e1 * Math.cos (2.0 *  phi) + 4.0 * e2 *
			cos (4.0 * phi) - 6.0 * e3 * Math.cos (6.0 *  phi);
		con1 = 2.0 * ml + c * (ml * ml + b) - 2.0 * a *  (c * ml + 1.0);
		con2 = eccent * sin2ph * (ml * ml + b - 2.0 * a * ml) / (2.0 * c);
		con3 = 2.0 * (a - ml) * (c * mlp - 2.0 / sin2ph) - 2.0 * mlp;
		dphi = con1 / (con2 + con3);
		phi += dphi;
		if (Math.abs(dphi) <= .0000000001 )
			phi_c = new Array(phi,c);
 	    	return(phi_c);
	}
	//alert("Latitude failed to converge,phi4z-conv");
	return(004);
}

/* Function to convert 2 digit alternate packed DMS format (+/-)DDDMMSS.SSS
   to 3 digit standard packed DMS format (+/-)DDDMMMSSS.SSS.
-----------------------------------------------------------------*/
function pakcz(pak) {
	sgna = ' ';
	if (pak < 0.0) 
		sgna = '-';
	con = Math.abs (pak);
	degs = ((con / 10000.0) + .001);
	con =  con  - degs * 10000;
	mins = ((con / 100.0) + .001);
	secs = con  - mins * 100;
	con = Number (degs) * 1000000.0 + Number (mins) * 1000.0 + secs;
	if (sgna == '-') 
		con = - con;
	return(con); 
}

/* Function to convert radians to 3 digit packed DMS format (+/-)DDDMMMSSS.SSS
----------------------------------------------------------------------------*/
function pakr2dm(pak) {
	sgna = ' ';
	pak *= R2D;
	if (pak < 0.0) 
		sgna = '-';
	con = Math.abs (pak);
	degs = (con);
	con =  (con  - degs) * 60;
	mins = con;
	secs = (con  - mins) * 60;
	con = Number (degs) * 1000000.0 + Number (mins) * 1000.0 + secs;
	if (sgna == '-') 
		con = - con;
	return(con); 
}

/* Function to compute the constant small t for use in the forward
   computations in the Lambert Conformal Conic and the Polar
   Stereographic projections.
--------------------------------------------------------------*/
function tsfnz(eccent,phi,sinphi) {
	con = eccent * sinphi;
	com = .5 * eccent; 
	con = Math.pow(((1.0 - con) / (1.0 + con)),com);
	return (Math.tan(.5 * (HALF_PI - phi))/con);
}


/* Function to return the sign of an argument
  ------------------------------------------*/
function sign(x) {
	if (x < 0.0)
    	return(-1);
	else
    	return(1);
}

/* Function to adjust a longitude angle to range from -180 to 180 radians
   added if statments 
  -----------------------------------------------------------------------*/
function adjust_lon(x)  {
	count = 0;
	for(;;) {
		if (Math.abs(x)<=PI)
			break;
		else
		if (( Math.abs(x / PI)) < 2)
			x = x-(sign(x) *TWO_PI);
		else
			if (( Math.abs(x / TWO_PI)) < MAXLONG) {
				x = x-(((x / TWO_PI))*TWO_PI);
			}
  			else
				if (( Math.abs(x / (MAXLONG * TWO_PI))) < MAXLONG) {
					x = x-(((x / (MAXLONG * TWO_PI))) * (TWO_PI * MAXLONG));
				}
				else
					if (( Math.abs(x / (DBLLONG * TWO_PI))) < MAXLONG) {
						x = x-(((x / (DBLLONG * TWO_PI))) * (TWO_PI * DBLLONG));
					}
					else
						x = x-(sign(x) * TWO_PI);
		count++;
		if (count > MAX_VAL)
			break;
	}

	return(x);
}

/* Functions to compute the constants e0, e1, e2, and e3 which are used
   in a series for calculating the distance along a meridian.  The
   input x represents the eccentricity squared.
----------------------------------------------------------------*/
function e0fn(x) {
	return(1.0-0.25*x*(1.0+x/16.0*(3.0+1.25*x)));
}

function e1fn(x) {
	return(0.375*x*(1.0+0.25*x*(1.0+0.46875*x)));
}

function e2fn(x) {
	return(0.05859375*x*x*(1.0+0.75*x));
}

function e3fn(x) {
	return(x*x*x*(35.0/3072.0));
}

/* Function to compute the constant e4 from the input of the eccentricity
   of the spheroid, x.  This constant is used in the Polar Stereographic
   projection.
--------------------------------------------------------------------*/
function e4fn(x) {
	con = 1.0 + x;
	com = 1.0 - x;
	return (Math.sqrt((Math.pow(con,con))*(Math.pow(com,com))));
}

/* Function computes the value of M which is the distance along a meridian
   from the Equator to latitude phi.
------------------------------------------------*/
function mlfn(e0,e1,e2,e3,phi) {
	return(e0*phi-e1*Math.sin(2.0*phi)+e2*Math.sin(4.0*phi)-e3*Math.sin(6.0*phi));
}

/* Function to calculate UTM zone number--NOTE Longitude entered in DEGREES!!!
  ---------------------------------------------------------------------------*/
function calc_utm_zone(lon) {
	return((((lon + 180.0) / 6.0) + 1.0));
}
