/*---------------------------------------------------------------------------
File: gctp_class.js

Description:
  Contains the gctp class which contains GCTP variables, reprojection
  functions, etc.

  +----------------------------------------------------------------------+
  | GCTP JS Edition Version 0.2.1                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 2004-2005 Aaron Koning and the University of           |
  |                         Northern British Columbia                    |
  +----------------------------------------------------------------------+
  | This file is subject to version 2 of the GNU General Public Licence, |
  | that is bundled with this package in the docs directory, and is also |
  | available through the world-wide-web at                              |
  | http://www.gnu.org/copyleft/gpl.html                                 |
  +----------------------------------------------------------------------+

History:
  2005-07-10	koninga		Created.
  2005-04-29    pisut@gistda.or.t.h  modified projection outside gctp class
                before use must declare "new gctp()" to initial projectios
                easy to use without totalgctp
---------------------------------------------------------------------------*/
// Projection Class
// Remodified by Pisut@GISTDA.org include group & desc
// Remove Projection Array Outside gctp class
// 
function projection(epsg, ellipsoid, detail, proj, r_maj, r_min, lat1, lat2, lon0, lat0, false_east, false_north, scale_fact, zone){
	// ----- VARIABLES -----------------
	// ---------------------------------
	this.epsg = epsg;
	this.ellipsoid = ellipsoid; //datum or spheroid name
	this.detail = detail; // description of zone or geographic
	this.proj = proj;
	this.r_maj = r_maj;
	this.r_min = r_min;
	this.lat1 = lat1;
	this.lat2 = lat2;
	this.lon0 = lon0;
	this.lat0 = lat0;
	this.false_east = false_east;
	this.false_north = false_north;
	this.scale_fact = scale_fact;
	this.zone = zone;
}

// GCTP class
// Remodified by Pisut@GISTDA.org include group & desc
function gctp() {
    this.Projs = new Array();
    this.InitialProjection = gctp_InitialProjection;
    this.InitialProjection();
	// ----- INTERFACE FUNCTIONS -------
	// ---------------------------------

	// Get the input projection
	this.getProjection = getProjection;
	function getProjection(epsg) {
		
		var projection;
		
		for(i=0;i<this.Projs.length;i++) {
			if(this.Projs[i].epsg == epsg) {
				return this.Projs[i];
			}
		}
		
		return false;
	}
	
	// Project the coordinates
	this.projectCoordinates = projectCoordinates;
	function projectCoordinates(inputX, inputY, inputProjEpsg, outputProjEpsg) {
		
		// Get Input Projection
		var inputProj = this.getProjection(inputProjEpsg);
		
		// Get Output Projection
		var outputProj = this.getProjection(outputProjEpsg);
		
		// Get Longlat coordinates
		var xy = this.getLonglatCoordinates(inputX, inputY, inputProj);
		var longlatX = xy[0];
		var longlatY = xy[1];
		
		// get the projected coordinates
		var xy = this.getProjectedCoordinates(longlatX, longlatY, outputProj);
		
		return xy;
	}

	// ----- UTILITY FUNCTIONS ---------
	// ---------------------------------

	// Translate projected coordinates to longlat coordinates
	this.getLonglatCoordinates = getLonglatCoordinates;
	function getLonglatCoordinates(inputX, inputY, inputProj) { 

		// Find the input projection and convert the projected coordinates
		switch(inputProj.proj) {
			case 'lamcc':
				var xy = this.projectLamccToLonglat(inputX, inputY, inputProj);
				break;
			case 'albers':
				var xy = this.projectAlbersToLonglat(inputX, inputY, inputProj);
				break;
			case 'utm':
				var xy = this.projectUtmToLonglat(inputX, inputY, inputProj);
				break;
			case 'longlat':
				var xy = new Array(inputX, inputY);
				break;
		}

		return xy;
	}


	// Translate longlat coordinates to projected coordinates
	this.getProjectedCoordinates = getProjectedCoordinates;
	function getProjectedCoordinates(longlatX, longlatY, outputProj) { 

		// Find the output projection and convert the longlat coordinates
		switch(outputProj.proj) {
			case 'lamcc':
				var xy = this.projectLonglatToLamcc(longlatX, longlatY, outputProj);
				break;
			case 'albers':
				var xy = this.projectLonglatToAlbers(longlatX, longlatY, outputProj);
				break;
			case 'utm':
				var xy = this.projectLonglatToUtm(longlatX, longlatY, outputProj);
				break;
			case 'longlat':
				var xy = new Array(longlatX, longlatY);
				break;
		}

		return xy;
	} // End function

	// PROJECTED TO LONGLAT FUNCTIONS -----

	// Project Lamcc coordinates to Long Lat coordinates
	this.projectLamccToLonglat = projectLamccToLonglat;
	function projectLamccToLonglat(projX, projY, proj) {
		// Initialize projection
		lamccinvint(proj.r_maj,proj.r_min,proj.lat1*D2R,proj.lat2*D2R,proj.lon0*D2R,proj.lat0*D2R,proj.false_east,proj.false_north);
		// Project coordinates
		var xy = lamccinv(projX,projY);
		xy[0] = xy[0]*R2D;
		xy[1] = xy[1]*R2D;
		
		return xy;
	}

	// Project Albers coordinates to Long Lat coordinates
	this.projectAlbersToLonglat = projectAlbersToLonglat;
	function projectAlbersToLonglat(projX, projY, proj) {
		// Initialize projection
		alberinvint(proj.r_maj,proj.r_min,proj.lat1*D2R,proj.lat2*D2R,proj.lon0*D2R,proj.lat0*D2R,proj.false_east,proj.false_north);
		// Project coordinates
		var xy = alberinv(projX,projY);
		xy[0] = xy[0]*R2D;
		xy[1] = xy[1]*R2D;

		return xy;
	}

	// Project Utm coordinates to Long Lat coordinates
	this.projectUtmToLonglat = projectUtmToLonglat;
	function projectUtmToLonglat(projX, projY, proj) {
		// Check for scale factor
		if(proj.scale_fact == false)
			var scale_fact = 0.9996;
		else
			var scale_fact = proj.scale_fact;
		// Initialize projection
		utminvint(proj.r_maj,proj.r_min,scale_fact,proj.zone);
		// Project coordinates
		var xy = utminv(projX,projY);
		xy[0] = xy[0]*R2D;
		xy[1] = xy[1]*R2D;
		
		return xy;

	}	

	// LONGLAT TO PROJECTED FUNCTIONS -----

	// Project Long Lat coordinates to Lamcc coordinates
	this.projectLonglatToLamcc = projectLonglatToLamcc;
	function projectLonglatToLamcc(longlatX, longlatY, proj) {	
		// Initialize projection
		lamccinvint(proj.r_maj,proj.r_min,proj.lat1*D2R,proj.lat2*D2R,proj.lon0*D2R,proj.lat0*D2R,proj.false_east,proj.false_north);
		// Project coordinates
		var xy = lamccfor(longlatX*D2R,longlatY*D2R);
		
		return xy;
	}
	
	// Project Long Lat coordinates to Albers coordinates
	this.projectLonglatToAlbers = projectLonglatToAlbers;
	function projectLonglatToAlbers(longlatX, longlatY, proj) {
		// Initialize projection
		alberinvint(proj.r_maj,proj.r_min,proj.lat1*D2R,proj.lat2*D2R,proj.lon0*D2R,proj.lat0*D2R,proj.false_east,proj.false_north);
		// Project coordinates
		var xy = alberfor(longlatX*D2R,longlatY*D2R);
		
		return xy;
	}

	// Project Long Lat coordinates to Albers coordinates
	this.projectLonglatToUtm = projectLonglatToUtm;
	function projectLonglatToUtm(longlatX, longlatY, proj) {
		// Check for scale factor
		if(proj.scale_fact == false)
			var scale_fact = 0.9996;
		else
			var scale_fact = proj.scale_fact;
		// Initialize projection
		utminvint(proj.r_maj,proj.r_min,scale_fact,proj.zone);
		// Project coordinates
		var xy = utmfor(longlatX*D2R,longlatY*D2R);
		
		return xy;
	}

} // End class
