/*******************************************************************************
NAME                            LAMBERT CONFORMAL CONIC 

PURPOSE:	Transforms input Easting and Northing to longitude and
		latitude for the Lambert Conformal Conic projection.  The
		Easting and Northing must be in meters.  The longitude
		and latitude values will be returned in radians.

PROGRAMMER				DATE				REASON
----------				----				------
T. Mittan				2-26-93				
A. Koning				2004-05-26			Ported to javascript

ALGORITHM REFERENCES

1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
*******************************************************************************/

/* Variables common to all subroutines in this code file
  -----------------------------------------------------*/
  var r_major;                /* major axis                   */
  var r_minor;                /* minor axis                   */
  var es;                     /* eccentricity squared         */
  var e;                      /* eccentricity                 */
  var center_lon;             /* center longituted            */
  var center_lat;             /* cetner latitude              */
  var ns;                     /* ratio of angle between meridian*/
  var f0;                     /* flattening of ellipsoid      */
  var rh;                     /* height above ellipsoid       */
  var false_easting;          /* x offset in meters           */
  var false_northing;		  /* y offset in meters		*/

/* Initialize the Lambert Conformal Conic projection
  ------------------------------------------------*/
function lamccinvint(r_maj,r_min,lat1,lat2,c_lon,c_lat,false_east,false_north) {
	/*---
	r_maj:				major axis			
	r_min:				minor axis			
	lat1:				first standard parallel	
	lat2:				second standard parallel	
	c_lon:				center longitude		
	c_lat:				center latitude		
	false_east:			x offset in meters		
	false_north:		y offset in meters		
	 
	sin_po:				sin value			
	cos_po:				cos value			
	con:				temporary sin value		
	ms1:				small m 1			
	ms2:				small m 2			
	temp:				temporary variable		
	ts0:				small t 0			
	ts1:				small t 1			
	ts2:				small t 2			
	---*/

	r_major = r_maj;
	r_minor = r_min;
	false_easting = false_east;
	false_northing = false_north;

	/* Standard Parallels cannot be equal and on opposite sides of the equator
	------------------------------------------------------------------------*/
	if (Math.abs(lat1+lat2) < EPSLN) {
		//alert("Equal Latitiudes for St. Parallels on opposite sides of equator, lamcc-inv");
		return(41);
	}

	temp = r_minor / r_major;
	es = 1.0 - Math.pow(temp,2);
	e = Math.sqrt(es);

	center_lon = c_lon;
	center_lat = c_lat;

	sincos_po = sincos(lat1);
	sin_po = sincos_po[0];
	cos_po = sincos_po[1];
	con = sin_po;

	ms1 = msfnz(e,sin_po,cos_po);
	ts1 = tsfnz(e,lat1,sin_po);
	
	sincos_po = sincos(lat2);
	sin_po = sincos_po[0];
	cos_po = sincos_po[1];

	ms2 = msfnz(e,sin_po,cos_po);
	ts2 = tsfnz(e,lat2,sin_po);
	sin_po = Math.sin(center_lat);
	ts0 = tsfnz(e,center_lat,sin_po);

	if (Math.abs(lat1 - lat2) > EPSLN)
		ns = Math.log(ms1/ms2) / Math.log(ts1/ts2);
	else
		ns = con;
	f0 = ms1 / (ns * Math.pow(ts1,ns));
	rh = r_major * f0 * Math.pow(ts0,ns);

	return true;
}

/* Lambert Conformal Conic inverse equations--mapping x,y to lat/long
  -----------------------------------------------------------------*/
function lamccinv(x, y) {
	/*---
	x:			 	(O) X projection coordinate 	
	y:			 	(O) Y projection coordinate 	
	
	rh1:			height above ellipsoid	
	con:			sign variable		
	ts:			 	small t			
	theta:			angle			
	flag:			error flag			
	---*/

	flag = 0;
	x -= false_easting;
	y = rh - y + false_northing;
	if (ns > 0) {
	    rh1 = Math.sqrt (x * x + y * y);
	    con = 1.0;
    }
 	else {
		rh1 = -Math.sqrt (x * x + y * y);
		con = -1.0;
    }
	theta = 0.0;
	if (rh1 != 0)
    	theta = Math.atan2((con * x),(con * y));
 	if ((rh1 != 0) || (ns > 0.0)) {
    	con = 1.0/ns;
    	ts = Math.pow((rh1/(r_major * f0)),con);
		lat = phi2z(e,ts,FLAG);
		if (FLAG != 0)
		   return(FLAG);
    }
	else
    	lat = -HALF_PI;
	lon = adjust_lon(theta/ns + center_lon);
	
	lonlat = new Array(lon,lat);
	return lonlat;
}
