/*******************************************************************************
NAME                            UNIVERSAL TRANSVERSE MERCATOR

PURPOSE:	Transforms input longitude and latitude to Easting and
		Northing for the Universal Transverse Mercator projection.
		The longitude and latitude must be in radians.  The Easting
		and Northing values will be returned in meters.

PROGRAMMER              DATE		REASON
----------              ----		------
D. Steinwand, EROS      Nov, 1991
T. Mittan				Mar, 1993
S. Nelson				Feb, 1995	Divided tmfor.c into two files, one
									for UTM (utmfor.c) and one for 
									TM (tmfor.c).  This was a
									necessary change to run forward
									projection conversions for both
									UTM and TM in the same process.
A. Koning				2005-07-08	Ported to javascript.

ALGORITHM REFERENCES

1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
*******************************************************************************/

/* Variables common to all subroutines in this code file
  -----------------------------------------------------*/
var r_major;		/* major axis 								*/
var r_minor;		/* minor axis 								*/
var scale_factor;	/* scale factor								*/
var lon_center;		/* Center longitude (projection center) 	*/
var lat_origin;		/* center latitude							*/
var e0,e1,e2,e3;	/* eccentricity constants					*/
var e,es,esp;		/* eccentricity constants					*/
var ml0;			/* small value m							*/
var false_northing;	/* y offset in meters						*/
var false_easting;	/* x offset in meters						*/
var ind;			/* spherical flag							*/

/* Initialize the Universal Transverse Mercator (UTM) projection
  -------------------------------------------------------------*/
function utmforint(r_maj,r_min,scale_fact,zone) {
	/*---
	r_maj:				major axis				
	r_min:				minor axis				
	scale_fact:			scale factor				
	zone:				zone number				
	temp;				temporary variable			
	---*/
	
	if ((Math.abs(zone) < 1) || (Math.abs(zone) > 60)) {
//			p_error("Illegal zone number","utm-forint");
		return;
	}
	
	r_major = r_maj;
	r_minor = r_min;
	scale_factor = scale_fact;
	lat_origin = 0.0;
	lon_center = ((6 * Math.abs(zone)) - 183) * D2R;
	false_easting = 500000.0;
	false_northing = (zone < 0) ? 10000000.0 : 0.0;

	temp = r_minor / r_major;
	es = 1.0 - (temp*temp);
	e = Math.sqrt(es);
	e0 = e0fn(es);
	e1 = e1fn(es);
	e2 = e2fn(es);
	e3 = e3fn(es);
	ml0 = r_major * mlfn(e0, e1, e2, e3, lat_origin);
	esp = es / (1.0 - es);

	if (es < .00001)
		ind = 1;

	/* Report parameters to the user
	  -----------------------------*/
//ptitle("UNIVERSAL TRANSVERSE MERCATOR (UTM)"); 
//genrpt_long(zone,   "Zone:     ");
//	radius2(r_major, r_minor);
//genrpt(scale_factor,"Scale Factor at C. Meridian:     ");
//	cenlonmer(lon_center);
	return;
}


/* Universal Transverse Mercator forward equations--mapping lat,long to x,y
   Note:  The algorithm for UTM is exactly the same as TM and therefore
	  if a change is implemented, also make the change to TMFOR.c
  -----------------------------------------------------------------------*/
function utmfor(lon, lat) {
	/*---
	lon:				(I) Longitude 		
	lat:			 	(I) Latitude 		
	x:			 		(O) X projection coordinate 	
	y:			 		(O) Y projection coordinate 	
	delta_lon:			Delta longitude (Given longitude - center 	
	theta:		 		angle					
	delta_theta:	 	adjusted longitude				
	sin_phi, cos_phi: 	sin and cos value				
	al, als:		 	temporary values				
	b:		 			temporary values				
	c, t, tq:	 		temporary values				
	con, n, ml:	 		cone constant, small m		
	---*/	

	/* Forward equations
	  -----------------*/
	delta_lon = adjust_lon(lon - lon_center);
//	sincos(lat, &sin_phi, &cos_phi);
	sincos_phi = sincos(lat);
	sin_phi = sincos_phi[0];
	cos_phi = sincos_phi[1];

	/* This part was in the fortran code and is for the spherical form 
	----------------------------------------------------------------*/
	if (ind != 0) {
		b = cos_phi * Math.sin(delta_lon);
		if ((Math.abs(Math.abs(b) - 1.0)) < .0000000001) {
			//p_error("Point projects into infinity","utm-for");
			return;
		}
		else {
			x = .5 * r_major * scale_factor * Math.log((1.0 + b)/(1.0 - b));
			con = Math.acos(cos_phi * Math.cos(delta_lon)/Math.sqrt(1.0 - b*b));
			if (lat < 0)
				con = - con;
			y = r_major * scale_factor * (con - lat_origin); 
			return;
		}
	}

	al  = cos_phi * delta_lon;
	als = (al*al);
	c   = esp * (cos_phi*cos_phi);
	tq  = Math.tan(lat);
	t   = (tq*tq);
	con = 1.0 - es * (sin_phi*sin_phi);
	n   = r_major / Math.sqrt(con);
	ml  = r_major * mlfn(e0, e1, e2, e3, lat);

	x  = scale_factor * n * al * (1.0 + als / 6.0 * (1.0 - t + c + als / 20.0 *
		(5.0 - 18.0 * t + (t*t) + 72.0 * c - 58.0 * esp))) + false_easting;

	y  = scale_factor * (ml - ml0 + n * tq * (als * (0.5 + als / 24.0 *
		(5.0 - t + 9.0 * c + 4.0 * (c*c) + als / 30.0 * (61.0 - 58.0 * t
		+ (t*t) + 600.0 * c - 330.0 * esp))))) + false_northing;

	xy = new Array(x,y);

	return(xy);
}
