/*******************************************************************************
NAME                            UNIVERSAL TRANSVERSE MERCATOR

PURPOSE:	Transforms input Easting and Northing to longitude and
		latitude for the Universal Transverse Mercator projection.
		The Easting and Northing must be in meters.  The longitude
		and latitude values will be returned in radians.

PROGRAMMER              DATE		REASON
----------              ----		------
D. Steinwand, EROS      Nov, 1991
T. Mittan				Mar, 1993
S. Nelson				Feb, 1995	Divided tminv.c into two files, one
									for UTM (utminv.c) and one for
									TM (tminv.c).  This was a necessary
									change to run inverse projection
									conversions for both UTM and TM
									in the same process.
A. Koning				2005-07-08	Ported to javascript.

ALGORITHM REFERENCES

1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
*******************************************************************************/

/* Variables common to all subroutines in this code file
  -----------------------------------------------------*/
var r_major;		/* major axis 								*/
var r_minor;		/* minor axis 								*/
var scale_factor;	/* scale factor								*/
var lon_center;		/* Center longitude (projection center) 	*/
var lat_origin;		/* center latitude							*/
var e0,e1,e2,e3;	/* eccentricity constants					*/
var e,es,esp;		/* eccentricity constants					*/
var ml0;			/* small value m							*/
var false_northing;	/* y offset in meters						*/
var false_easting;	/* x offset in meters						*/
var ind;			/* spherical flag							*/

/* Initialize the Universal Transverse Mercator (UTM) projection
  -------------------------------------------------------------*/
function utminvint(r_maj,r_min,scale_fact,zone) {
	/*---
	r_maj:				major axis				
	r_min:				minor axis				
	scale_fact:			scale factor				
	zone:				zone number				
	temp;				temporary variable			
	---*/

	if ((Math.abs(zone) < 1) || (Math.abs(zone) > 60)) {
		//p_error("Illegal zone number","utm-invint");
		return;
	}
	
	r_major = r_maj;
	r_minor = r_min;
	scale_factor = scale_fact;
	lat_origin = 0.0;
	lon_center = ((6 * Math.abs(zone)) - 183) * D2R;
	false_easting = 500000.0;
	false_northing = (zone < 0) ? 10000000.0 : 0.0;

	temp = r_minor / r_major;
	es = 1.0 - (temp*temp);
	e = Math.sqrt(es);
	e0 = e0fn(es);
	e1 = e1fn(es);
	e2 = e2fn(es);
	e3 = e3fn(es);
	ml0 = r_major * mlfn(e0, e1, e2, e3, lat_origin);
	esp = es / (1.0 - es);

	if (es < .00001)
		ind = 1;
	else 
		ind = 0;

	/* Report parameters to the user
	  -----------------------------*/
//ptitle("UNIVERSAL TRANSVERSE MERCATOR (UTM)"); 
//genrpt_long(zone,   "Zone:     ");
//	radius2(r_major, r_minor);
//genrpt(scale_factor,"Scale Factor at C. Meridian:     ");
//	cenlonmer(lon_center);
	return;
}

/* Universal Transverse Mercator inverse equations--mapping x,y to lat,long 
   Note:  The algorithm for UTM is exactly the same as TM and therefore
	  if a change is implemented, also make the change to TMINV.c
  -----------------------------------------------------------------------*/
function utminv(x, y) {
	/*---
	x:			 				(I) X projection coordinate 	
	y:			 				(I) Y projection coordinate 	
	lon:						(O) Longitude 		
	lat:			 			(O) Latitude 		
	con,phi:					temporary angles
	delta_phi:					difference between longitudes
	i:							counter variable
	sin_phi, cos_phi, tan_phi:	sin cos and tangent values
	c, cs, t, ts, n, r, d, ds:	temporary variables
	f, h, g, temp:				temporary variables
	---*/
	
	var max_iter = 6;			/* maximun number of iterations	*/

	/* fortran code for spherical form 
	--------------------------------*/
	if (ind != 0) {
		f = Math.exp(x/(r_major * scale_factor));
		g = .5 * (f - 1/f);
		temp = lat_origin + y/(r_major * scale_factor);
		h = Math.cos(temp);
		con = Math.sqrt((1.0 - h * h)/(1.0 + g * g));
   		lat = asinz(con);
   		if (temp < 0)
			lat = -lat;
		if ((g == 0) && (h == 0)) {
			lon = lon_center;
			lonlat = new Array(lon,lat);
			return lonlat;
		}
		else {
			lon = adjust_lon(Math.atan2(g,h) + lon_center);
			lonlat = new Array(lon,lat);
			return lonlat;
		}
	}

	/* Inverse equations
	  -----------------*/
	x = x - false_easting;
	y = y - false_northing;

	con = (ml0 + y / scale_factor) / r_major;
	phi = con;
	for (i=0;;i++) {
		delta_phi=((con + e1 * Math.sin(2.0*phi) - e2 * Math.sin(4.0*phi) + e3 * Math.sin(6.0*phi))
               / e0) - phi;
		phi += delta_phi;
		if (Math.abs(delta_phi) <= EPSLN) break;
		if (i >= max_iter) { 
			//p_error("Latitude failed to converge","UTM-INVERSE"); 
			return;
		}
	}
	if (Math.abs(phi) < HALF_PI) {
		sincos_phi = sincos(phi);
		sin_phi = sincos_phi[0];
		cos_phi = sincos_phi[1];
		
		tan_phi = Math.tan(phi);
		c    = esp * (cos_phi*cos_phi);
		cs   = (c*c);
		t    = (tan_phi*tan_phi);
		ts   = (t*t);
		con  = 1.0 - es * (sin_phi*sin_phi); 
		n    = r_major / Math.sqrt(con);
		r    = n * (1.0 - es) / con;
		d    = x / (n * scale_factor);
		ds   = (d*d);
		lat = phi - (n * tan_phi * ds / r) * (0.5 - ds / 24.0 * (5.0 + 3.0 * t + 
          10.0 * c - 4.0 * cs - 9.0 * esp - ds / 30.0 * (61.0 + 90.0 * t +
          298.0 * c + 45.0 * ts - 252.0 * esp - 3.0 * cs)));
		lon = adjust_lon(lon_center + (d * (1.0 - ds / 6.0 * (1.0 + 2.0 * t +
          c - ds / 20.0 * (5.0 - 2.0 * c + 28.0 * t - 3.0 * cs + 8.0 * esp +
          24.0 * ts))) / cos_phi));
	}
	else {
		lat = HALF_PI * sign(y);
		lon = lon_center;
	}
	
	lonlat = new Array(lon,lat);
	return lonlat;
}
